@@ -1,5 +1,5 @@
 require 'cgi'
-require 'json'
+require 'multi_json'
 require 'excon'
 require 'tempfile'
 require 'base64'
@@ -14,7 +14,9 @@ require 'open-uri'
 require 'excon/middlewares/hijack'
 Excon.defaults[:middlewares].unshift Excon::Middleware::Hijack
 
-# The top-level module for this gem. It's purpose is to hold global
+Excon.defaults[:middlewares] << Excon::Middleware::RedirectFollower
+
+# The top-level module for this gem. Its purpose is to hold global
 # configuration variables that are used as defaults in other classes.
 module Docker
   attr_accessor :creds, :logger
@@ -23,6 +25,7 @@ module Docker
   require 'docker/connection'
   require 'docker/base'
   require 'docker/container'
+  require 'docker/network'
   require 'docker/event'
   require 'docker/exec'
   require 'docker/image'
@@ -30,6 +33,7 @@ module Docker
   require 'docker/messages'
   require 'docker/util'
   require 'docker/version'
+  require 'docker/volume'
   require 'docker/rake_task' if defined?(Rake::Task)
 
   def default_socket_url
@@ -102,35 +106,41 @@ module Docker
 
   # Get the version of Go, Docker, and optionally the Git commit.
   def version(connection = self.connection)
-    Util.parse_json(connection.get('/version'))
+    connection.version
   end
 
   # Get more information about the Docker server.
   def info(connection = self.connection)
-    Util.parse_json(connection.get('/info'))
+    connection.info
+  end
+
+  # Ping the Docker server.
+  def ping(connection = self.connection)
+    connection.ping
+  end
+
+  # Determine if the server is podman or docker.
+  def podman?(connection = self.connection)
+    connection.podman?
+  end
+
+  # Determine if the session is rootless.
+  def rootless?(connection = self.connection)
+    connection.rootless?
   end
 
   # Login to the Docker registry.
   def authenticate!(options = {}, connection = self.connection)
-    creds = options.to_json
-    connection.post('/auth', {}, :body => creds)
+    creds = MultiJson.dump(options)
+    connection.post('/auth', {}, body: creds)
     @creds = creds
     true
   rescue Docker::Error::ServerError, Docker::Error::UnauthorizedError
     raise Docker::Error::AuthenticationError
   end
 
-  # When the correct version of Docker is installed, returns true. Otherwise,
-  # raises a VersionError.
-  def validate_version!
-    Docker.info
-    true
-  rescue Docker::Error::DockerError
-    raise Docker::Error::VersionError, "Expected API Version: #{API_VERSION}"
-  end
-
   module_function :default_socket_url, :env_url, :url, :url=, :env_options,
                   :options, :options=, :creds, :creds=, :logger, :logger=,
                   :connection, :reset!, :reset_connection!, :version, :info,
-                  :authenticate!, :validate_version!, :ssl_options
+                  :ping, :podman?, :rootless?, :authenticate!, :ssl_options
 end
