@@ -3,20 +3,33 @@
 class Docker::Container
   include Docker::Base
 
+  # Update the @info hash, which is the only mutable state in this object.
+  #   e.g. if you would like a live status from the #info hash, call #refresh! first.
+  def refresh!
+    other = Docker::Container.all({all: true}, connection).find { |c|
+      c.id.start_with?(self.id) || self.id.start_with?(c.id)
+    }
+
+    info.merge!(self.json)
+    other && info.merge!(other.info) { |key, info_value, other_value| info_value }
+    self
+  end
+
   # Return a List of Hashes that represents the top running processes.
   def top(opts = {})
+    format = opts.delete(:format) { :array }
     resp = Docker::Util.parse_json(connection.get(path_for(:top), opts))
     if resp['Processes'].nil?
-      []
+      format == :array ? [] : {}
     else
-      resp['Processes'].map { |ary| Hash[resp['Titles'].zip(ary)] }
+      format == :array ? resp['Processes'].map { |ary| Hash[resp['Titles'].zip(ary)] } : resp
     end
   end
 
   # Wait for the current command to finish executing. Default wait time is
   # `Excon.options[:read_timeout]`.
   def wait(time = nil)
-    excon_params = { :read_timeout => time, :idempotent => true }
+    excon_params = { :read_timeout => time }
     resp = connection.post(path_for(:wait), nil, excon_params)
     Docker::Util.parse_json(resp)
   end
@@ -39,17 +52,17 @@ class Docker::Container
   # @param options [Hash] The options to pass to Docker::Exec
   #
   # @return [Docker::Exec] The Exec instance
-  def exec(command, opts = {}, &block)
+  def exec(command, options = {}, &block)
     # Establish values
-    tty = opts.delete(:tty) || false
-    detach = opts.delete(:detach) || false
-    user = opts.delete(:user)
-    stdin = opts.delete(:stdin)
-    stdout = opts.delete(:stdout) || !detach
-    stderr = opts.delete(:stderr) || !detach
+    tty = options.delete(:tty) || false
+    detach = options.delete(:detach) || false
+    user = options.delete(:user)
+    stdin = options.delete(:stdin)
+    stdout = options.delete(:stdout) || !detach
+    stderr = options.delete(:stderr) || !detach
+    wait = options.delete(:wait)
 
-    # Create Exec Instance
-    instance = Docker::Exec.create(
+    opts = {
       'Container' => self.id,
       'User' => user,
       'AttachStdin' => !!stdin,
@@ -57,12 +70,19 @@ class Docker::Container
       'AttachStderr' => stderr,
       'Tty' => tty,
       'Cmd' => command
+    }.merge(options)
+
+    # Create Exec Instance
+    instance = Docker::Exec.create(
+      opts,
+      self.connection
     )
 
     start_opts = {
       :tty => tty,
       :stdin => stdin,
-      :detach => detach
+      :detach => detach,
+      :wait => wait
     }
 
     if detach
@@ -80,7 +100,7 @@ class Docker::Container
   end
 
   # Attach to a container's standard streams / logs.
-  def attach(options = {}, &block)
+  def attach(options = {}, excon_params = {}, &block)
     stdin = options.delete(:stdin)
     tty   = options.delete(:tty)
 
@@ -90,8 +110,6 @@ class Docker::Container
     # Creates list to store stdout and stderr messages
     msgs = Docker::Messages.new
 
-    excon_params = {}
-
     if stdin
       # If attaching to stdin, we must hijack the underlying TCP connection
       # so we can stream stdin to the remote Docker process
@@ -117,10 +135,10 @@ class Docker::Container
     # Based on the link, the config passed as run, needs to be passed as the
     # body of the post so capture it, remove from the options, and pass it via
     # the post body
-    config = options.delete('run')
-    hash = Docker::Util.parse_json(connection.post('/commit',
-                                                   options,
-                                                   :body => config.to_json))
+    config = MultiJson.dump(options.delete('run'))
+    hash = Docker::Util.parse_json(
+      connection.post('/commit', options, body: config)
+    )
     Docker::Image.send(:new, self.connection, hash)
   end
 
@@ -141,24 +159,47 @@ class Docker::Container
     connection.get(path_for(:logs), opts)
   end
 
+  def stats(options = {})
+    if block_given?
+      options[:read_timeout] ||= 10
+      options[:idempotent] ||= false
+      parser = lambda do |chunk, remaining_bytes, total_bytes|
+        yield Docker::Util.parse_json(chunk)
+      end
+      begin
+        connection.get(path_for(:stats), nil, {response_block: parser}.merge(options))
+      rescue Docker::Error::TimeoutError
+        # If the container stops, the docker daemon will hold the connection
+        # open forever, but stop sending events.
+        # So this Timeout indicates the stream is over.
+      end
+    else
+      Docker::Util.parse_json(connection.get(path_for(:stats), {stream: 0}.merge(options)))
+    end
+  end
+
   def rename(new_name)
     query = {}
     query['name'] = new_name
     connection.post(path_for(:rename), query)
   end
 
+  def update(opts)
+    connection.post(path_for(:update), {}, body: MultiJson.dump(opts))
+  end
+
   def streaming_logs(opts = {}, &block)
-    stack_size = opts.delete('stack_size') || -1
+    stack_size = opts.delete('stack_size') || opts.delete(:stack_size) || -1
     tty = opts.delete('tty') || opts.delete(:tty) || false
     msgs = Docker::MessagesStack.new(stack_size)
-    excon_params = {response_block: Docker::Util.attach_for(block, msgs, tty)}
+    excon_params = {response_block: Docker::Util.attach_for(block, msgs, tty), idempotent: false}
 
     connection.get(path_for(:logs), opts, excon_params)
     msgs.messages.join
   end
 
   def start!(opts = {})
-    connection.post(path_for(:start), {}, :body => opts.to_json)
+    connection.post(path_for(:start), {}, body: MultiJson.dump(opts))
     self
   end
 
@@ -183,8 +224,18 @@ class Docker::Container
     define_method(:"#{method}!") do |opts = {}|
       timeout = opts.delete('timeout')
       query = {}
-      query['t'] = timeout if timeout
-      connection.post(path_for(method), query, :body => opts.to_json)
+      request_options = {
+        :body => MultiJson.dump(opts)
+      }
+      if timeout
+        query['t'] = timeout
+        # Ensure request does not timeout before Docker timeout
+        request_options.merge!(
+          read_timeout: timeout.to_i + 5,
+          write_timeout: timeout.to_i + 5
+        )
+      end
+      connection.post(path_for(method), query, request_options)
       self
     end
 
@@ -215,27 +266,74 @@ class Docker::Container
     end
   end
 
-  def copy(path, &block)
-    connection.post(path_for(:copy), {},
-      :body => { "Resource" => path }.to_json,
+  def archive_out(path, &block)
+    connection.get(
+      path_for(:archive),
+      { 'path' => path },
       :response_block => block
     )
     self
   end
 
+  def archive_in(inputs, output_path, opts = {})
+    file_hash = Docker::Util.file_hash_from_paths([*inputs])
+    tar = StringIO.new(Docker::Util.create_tar(file_hash))
+    archive_in_stream(output_path, opts) do
+      tar.read(Excon.defaults[:chunk_size]).to_s
+    end
+  end
+
+  def archive_in_stream(output_path, opts = {}, &block)
+    overwrite = opts[:overwrite] || opts['overwrite'] || false
+
+    connection.put(
+      path_for(:archive),
+      { 'path' => output_path, 'noOverwriteDirNonDir' => !overwrite },
+      :headers => {
+        'Content-Type' => 'application/x-tar'
+      },
+      &block
+    )
+    self
+  end
+
+  def read_file(path)
+    content = StringIO.new
+    archive_out(path) do |chunk|
+      content.write chunk
+    end
+
+    content.rewind
+
+    Gem::Package::TarReader.new(content) do |tar|
+      tar.each do |tarfile|
+        return tarfile.read
+      end
+    end
+  end
+
+  def store_file(path, file_content)
+    output_io = StringIO.new(
+      Docker::Util.create_tar(
+        path => file_content
+      )
+    )
+
+    archive_in_stream("/", overwrite: true) { output_io.read }
+  end
+
   # Create a new Container.
   def self.create(opts = {}, conn = Docker.connection)
-    name = opts.delete('name')
-    query = {}
-    query['name'] = name if name
-    resp = conn.post('/containers/create', query, :body => opts.to_json)
+    query = opts.select {|key| ['name', :name].include?(key) }
+    clean_opts = opts.reject {|key| ['name', :name].include?(key) }
+    resp = conn.post('/containers/create', query, :body => MultiJson.dump(clean_opts))
     hash = Docker::Util.parse_json(resp) || {}
     new(conn, hash)
   end
 
   # Return the container with specified ID
   def self.get(id, opts = {}, conn = Docker.connection)
-    container_json = conn.get("/containers/#{URI.encode(id)}/json", opts)
+    container_json = conn.get("/containers/#{id}/json", opts)
     hash = Docker::Util.parse_json(container_json) || {}
     new(conn, hash)
   end
@@ -246,6 +344,12 @@ class Docker::Container
     hashes.map { |hash| new(conn, hash) }
   end
 
+  # Prune images
+  def self.prune(conn = Docker.connection)
+    conn.post("/containers/prune", {})
+    nil
+  end
+
   # Convenience method to return the path for a particular resource.
   def path_for(resource)
     "/containers/#{self.id}/#{resource}"
