@@ -17,6 +17,9 @@ module Docker::Error
   # Raised when a request returns a 404.
   class NotFoundError < DockerError; end
 
+  # Raised when a request returns a 409.
+  class ConflictError < DockerError; end
+
   # Raised when a request returns a 500.
   class ServerError < DockerError; end
 
