@@ -6,17 +6,17 @@ class Docker::Image
   # an Image. This will not modify the Image, but rather create a new Container
   # to run the Image. If the image has an embedded config, no command is
   # necessary, but it will fail with 500 if no config is saved with the image
-  def run(cmd=nil)
-    opts = { 'Image' => self.id }
+  def run(cmd = nil, options = {})
+    opts = {'Image' => self.id}.merge(options)
     opts["Cmd"] = cmd.is_a?(String) ? cmd.split(/\s+/) : cmd
     begin
       Docker::Container.create(opts, connection)
                        .tap(&:start!)
-    rescue ServerError => ex
+    rescue ServerError, ClientError => ex
       if cmd
         raise ex
       else
-        raise ServerError, "No command specified."
+        raise ex, "No command specified."
       end
     end
   end
@@ -65,7 +65,16 @@ class Docker::Image
 
   # Remove the Image from the server.
   def remove(opts = {})
-    name = opts.delete(:name) || self.id
+    name = opts.delete(:name)
+
+    unless name
+      if ::Docker.podman?
+        name = self.id.split(':').last
+      else
+        name = self.id
+      end
+    end
+
     connection.delete("/images/#{name}", opts)
   end
   alias_method :delete, :remove
@@ -89,6 +98,11 @@ class Docker::Image
     self.class.save(self.id, filename, connection)
   end
 
+  # Save the image as a tarball to an IO object.
+  def save_stream(opts = {}, &block)
+    self.class.save_stream(self.id, opts, connection, &block)
+  end
+
   # Update the @info hash, which is the only mutable state in this object.
   def refresh!
     img = Docker::Image.all({:all => true}, connection).find { |image|
@@ -103,7 +117,7 @@ class Docker::Image
 
     # Create a new Image.
     def create(opts = {}, creds = nil, conn = Docker.connection, &block)
-      credentials = creds.nil? ? Docker.creds : creds.to_json
+      credentials = creds.nil? ? Docker.creds : MultiJson.dump(creds)
       headers = credentials && Docker::Util.build_auth_header(credentials) || {}
       body = ''
       conn.post(
@@ -111,19 +125,33 @@ class Docker::Image
         opts,
         :headers => headers,
         :response_block => response_block(body, &block)
-      )
-      json = Docker::Util.fix_json(body)
-      image = json.reverse_each.find { |el| el && el.key?('id') }
-      new(conn, 'id' => image && image.fetch('id'), :headers => headers)
+        )
+      # NOTE: see associated tests for why we're looking at image#end_with?
+      image = opts['fromImage'] || opts[:fromImage]
+      tag = opts['tag'] || opts[:tag]
+      image = "#{image}:#{tag}" if tag && !image.end_with?(":#{tag}")
+      get(image, {}, conn)
     end
 
     # Return a specific image.
     def get(id, opts = {}, conn = Docker.connection)
-      image_json = conn.get("/images/#{URI.encode(id)}/json", opts)
+      image_json = conn.get("/images/#{id}/json", opts)
       hash = Docker::Util.parse_json(image_json) || {}
       new(conn, hash)
     end
 
+    # Delete a specific image
+    def remove(id, opts = {}, conn = Docker.connection)
+      conn.delete("/images/#{id}", opts)
+    end
+    alias_method :delete, :remove
+
+    # Prune images
+    def prune(conn = Docker.connection)
+      conn.post("/images/prune", {})
+    end
+
+
     # Save the raw binary representation or one or more Docker images
     #
     # @param names [String, Array#String] The image(s) you wish to save
@@ -134,27 +162,49 @@ class Docker::Image
     # representation of the binary data. If the filename is not nil, then
     # return nil.
     def save(names, filename = nil, conn = Docker.connection)
-      # By using compare_by_identity we can create a Hash that has
-      # the same key multiple times.
-      query = {}
-      query.compare_by_identity
-      Array(names).each do |name|
-        query['names'.dup] = URI.encode(name)
-      end
-
       if filename
-        file = File.open(filename, 'wb')
-        conn.get(
-          '/images/get', query,
-          :response_block => response_block_for_save(file)
-        )
-        file.close
+        File.open(filename, 'wb') do |file|
+          save_stream(names, {}, conn, &response_block_for_save(file))
+        end
         nil
       else
-        conn.get('/images/get', query)
+        string = ''
+        save_stream(names, {}, conn, &response_block_for_save(string))
+        string
       end
     end
 
+    # Stream the contents of Docker image(s) to a block.
+    #
+    # @param names [String, Array#String] The image(s) you wish to save
+    # @param conn [Docker::Connection] The Docker connection to use
+    # @yield chunk [String] a chunk of the Docker image(s).
+    def save_stream(names, opts = {}, conn = Docker.connection, &block)
+      # By using compare_by_identity we can create a Hash that has
+      # the same key multiple times.
+      query = {}.tap(&:compare_by_identity)
+      Array(names).each { |name| query['names'.dup] = name }
+      conn.get(
+        '/images/get',
+        query,
+        opts.merge(:response_block => block)
+      )
+      nil
+    end
+
+    # Load a tar Image
+    def load(tar, opts = {}, conn = Docker.connection, creds = nil, &block)
+       headers = build_headers(creds)
+       io = tar.is_a?(String) ? File.open(tar, 'rb') : tar
+       body = ""
+       conn.post(
+         '/images/load',
+         opts,
+         :headers => headers,
+         :response_block => response_block(body, &block)
+       ) { io.read(Excon.defaults[:chunk_size]).to_s }
+    end
+
     # Check if an image exists.
     def exist?(id, opts = {}, conn = Docker.connection)
       get(id, opts, conn)
@@ -171,8 +221,14 @@ class Docker::Image
 
     # Given a query like `{ :term => 'sshd' }`, queries the Docker Registry for
     # a corresponding Image.
-    def search(query = {}, connection = Docker.connection)
-      body = connection.get('/images/search', query)
+    def search(query = {}, connection = Docker.connection, creds = nil)
+      credentials = creds.nil? ? Docker.creds : creds.to_json
+      headers = credentials && Docker::Util.build_auth_header(credentials) || {}
+      body = connection.get(
+        '/images/search',
+        query,
+        :headers => headers,
+      )
       hashes = Docker::Util.parse_json(body) || []
       hashes.map { |hash| new(connection, 'id' => hash['name']) }
     end
@@ -180,7 +236,16 @@ class Docker::Image
     # Import an Image from the output of Docker::Container#export. The first
     # argument may either be a File or URI.
     def import(imp, opts = {}, conn = Docker.connection)
-      open(imp) do |io|
+      require 'open-uri'
+
+      # This differs after Ruby 2.4
+      if URI.public_methods.include?(:open)
+        munged_open = URI.method(:open)
+      else
+        munged_open = self.method(:open)
+      end
+
+      munged_open.call(imp) do |io|
         import_stream(opts, conn) do
           io.read(Excon.defaults[:chunk_size]).to_s
         end
