@@ -1,6 +1,11 @@
+require 'set'
+
 # This module holds shared logic that doesn't really belong anywhere else in the
 # gem.
 module Docker::Util
+  # http://www.tldp.org/LDP/GNU-Linux-Tools-Summary/html/x11655.htm#STANDARD-WILDCARDS
+  GLOB_WILDCARDS = /[\?\*\[\{\]\}]/
+
   include Docker::Error
 
   module_function
@@ -99,8 +104,8 @@ module Docker::Util
   end
 
   def parse_json(body)
-    JSON.parse(body) unless body.nil? || body.empty? || (body == 'null')
-  rescue JSON::ParserError => ex
+    MultiJson.load(body) unless body.nil? || body.empty? || (body == 'null')
+  rescue MultiJson::ParseError => ex
     raise UnexpectedResponseError, ex.message
   end
 
@@ -119,8 +124,12 @@ module Docker::Util
   def create_tar(hash = {})
     output = StringIO.new
     Gem::Package::TarWriter.new(output) do |tar|
-      hash.each do |file_name, input|
-        tar.add_file(file_name, 0640) { |tar_file| tar_file.write(input) }
+      hash.each do |file_name, file_details|
+        permissions = file_details.is_a?(Hash) ? file_details[:permissions] : 0640
+        tar.add_file(file_name, permissions) do |tar_file|
+          content = file_details.is_a?(Hash) ? file_details[:content] : file_details
+          tar_file.write(content)
+        end
       end
     end
     output.tap(&:rewind).string
@@ -135,9 +144,37 @@ module Docker::Util
     File.new(tempfile.path, 'r')
   end
 
+
+  # return the set of files that form the docker context
+  # implement this logic https://docs.docker.com/engine/reference/builder/#dockerignore-file
+  def docker_context(directory)
+    all_files = glob_all_files(File.join(directory, "**/*"))
+    dockerignore = File.join(directory, '.dockerignore')
+    return all_files unless all_files.include?(dockerignore)
+
+    # Iterate over valid lines, starting with the initial glob as working set
+    File
+      .read(dockerignore)                # https://docs.docker.com/engine/reference/builder/#dockerignore-file
+      .each_line                         # "a newline-separated list of patterns"
+      .map(&:strip)                      # "A preprocessing step removes leading and trailing whitespace"
+      .reject(&:empty?)                  # "Lines that are blank after preprocessing are ignored"
+      .reject { |p| p.start_with?('#') } # "if [a line starts with `#`], then this line is considered as a comment"
+      .each_with_object(Set.new(all_files)) do |p, working_set|
+        # determine the pattern (p) and whether it is to be added or removed from context
+        add = p.start_with?("!")
+        # strip leading "!" from pattern p, then prepend the base directory
+        matches = dockerignore_compatible_glob(File.join(directory, add ? p[1..-1] : p))
+        # add or remove the matched items as indicated in the ignore file
+        add ? working_set.merge(matches) : working_set.replace(working_set.difference(matches))
+      end
+    .to_a
+  end
+
   def create_relative_dir_tar(directory, output)
     Gem::Package::TarWriter.new(output) do |tar|
-      Find.find(directory) do |prefixed_file_name|
+      files = docker_context(directory)
+
+      files.each do |prefixed_file_name|
         stat = File.stat(prefixed_file_name)
         next unless stat.file?
 
@@ -201,18 +238,29 @@ module Docker::Util
       basename = File.basename(local_path)
       if File.directory?(local_path)
         tar = create_dir_tar(local_path)
-        file_hash[basename] = tar.read
+        file_hash[basename] = {
+          content: tar.read,
+          permissions: filesystem_permissions(local_path)
+        }
         tar.close
         FileUtils.rm(tar.path)
       else
-        file_hash[basename] = File.read(local_path)
+        file_hash[basename] = {
+          content: File.read(local_path, mode: 'rb'),
+          permissions: filesystem_permissions(local_path)
+        }
       end
     end
   end
 
+  def filesystem_permissions(path)
+    mode = sprintf("%o", File.stat(path).mode)
+    mode[(mode.length - 3)...mode.length].to_i(8)
+  end
+
   def build_auth_header(credentials)
-    credentials = credentials.to_json if credentials.is_a?(Hash)
-    encoded_creds = Base64.encode64(credentials).gsub(/\n/, '')
+    credentials = MultiJson.dump(credentials) if credentials.is_a?(Hash)
+    encoded_creds = Base64.urlsafe_encode64(credentials)
     {
       'X-Registry-Auth' => encoded_creds
     }
@@ -220,22 +268,41 @@ module Docker::Util
 
   def build_config_header(credentials)
     if credentials.is_a?(String)
-      credentials = JSON.parse(credentials, symbolize_names: true)
+      credentials = MultiJson.load(credentials, symbolize_keys: true)
     end
-    header = {
-      "configs" => {
-        credentials[:serveraddress].to_s => {
-          "username" => credentials[:username].to_s,
-          "password" => credentials[:password].to_s,
-          "email" => credentials[:email].to_s
-        }
-      }
-    }.to_json
 
-    encoded_header = Base64.encode64(header).gsub(/\n/, '')
+    header = MultiJson.dump(
+      credentials[:serveraddress].to_s => {
+        'username' => credentials[:username].to_s,
+        'password' => credentials[:password].to_s,
+        'email' => credentials[:email].to_s
+      }
+    )
+
+    encoded_header = Base64.urlsafe_encode64(header)
 
     {
       'X-Registry-Config' => encoded_header
     }
   end
+
+  # do a directory glob that matches .dockerignore behavior
+  # specifically: matched directories are considered a recursive match
+  def dockerignore_compatible_glob(pattern)
+    begin
+      some_dirs, some_files = glob_all_files(pattern).partition { |f| File.directory?(f) }
+      # since all directories will be re-processed with a /**/* glob, we can preemptively
+      # eliminate any whose parent directory is already in this set.  This saves significant time.
+      some_files + some_dirs.reject { |d| some_dirs.any? { |pd| d.start_with?(pd) && d != pd } }
+    end.each_with_object(Set.new) do |f, acc|
+      # expand any directories by globbing; flatten results
+      acc.merge(File.directory?(f) ? glob_all_files("#{f}/**/*") : [f])
+    end
+  end
+
+  def glob_all_files(pattern)
+    # globs of "a_dir/**/*" can return "a_dir/.", so explicitly reject those
+    (Dir.glob(pattern, File::FNM_DOTMATCH) - ['..', '.']).reject { |p| p.end_with?("/.") }
+  end
+
 end
