@@ -1,7 +1,7 @@
 require 'spec_helper'
 
-# WARNING if you're re-recording any of these VCRs, you must be running the
-# Docker daemon and have the base Image pulled.
+SingleCov.covered! uncovered: 39
+
 describe Docker::Container do
   describe '#to_s' do
     subject {
@@ -25,12 +25,12 @@ describe Docker::Container do
 
   describe '#json' do
     subject {
-      described_class.create('Cmd' => %w[true], 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => %w[true], 'Image' => 'debian:stable')
     }
     let(:description) { subject.json }
     after(:each) { subject.remove }
 
-    it 'returns the description as a Hash', :vcr do
+    it 'returns the description as a Hash' do
       expect(description).to be_a Hash
       expect(description['Id']).to start_with(subject.id)
     end
@@ -40,7 +40,7 @@ describe Docker::Container do
     let(:options) { {} }
     subject do
       described_class.create(
-        {'Cmd' => ['/bin/bash', '-lc', 'echo hello'], 'Image' => 'debian:wheezy'}.merge(options)
+        {'Cmd' => ['/bin/bash', '-lc', 'echo hello'], 'Image' => 'debian:stable'}.merge(options)
       )
     end
 
@@ -49,16 +49,16 @@ describe Docker::Container do
 
     context 'when not selecting any stream' do
       let(:non_destination) { subject.streaming_logs }
-      it 'raises a client error', :vcr do
+      it 'raises a client error' do
         expect { non_destination }.to raise_error(Docker::Error::ClientError)
       end
     end
 
     context 'when selecting stdout' do
       let(:stdout) { subject.streaming_logs(stdout: 1) }
-      it 'returns blank logs', :vcr do
+      it 'returns blank logs' do
         expect(stdout).to be_a String
-        expect(stdout).to eq "hello\n"
+        expect(stdout).to match("hello")
       end
     end
 
@@ -66,39 +66,79 @@ describe Docker::Container do
       let(:options) { { 'Tty' => true } }
 
       let(:output) { subject.streaming_logs(stdout: 1, tty: 1) }
-      it 'returns `hello`', :vcr do
+      it 'returns `hello`' do
         expect(output).to be_a(String)
-        expect(output).to eq("hello\n")
+        expect(output).to match("hello")
       end
     end
 
     context 'when passing a block' do
       let(:lines) { [] }
       let(:output) { subject.streaming_logs(stdout: 1, follow: 1) { |s,c| lines << c } }
-      it 'returns `hello`', :vcr do
+      it 'returns `hello`' do
         expect(output).to be_a(String)
-        expect(output).to eq("hello\n")
-        expect(lines).to eq(["hello\n"])
+        expect(output).to match("hello")
+        expect(lines.join).to match("hello")
+      end
+    end
+  end
+
+  describe '#stats', :docker_1_9 do
+    after(:each) do
+      subject.wait
+      subject.remove
+    end
+
+    context "when requesting container stats" do
+      subject {
+        described_class.create('Cmd' => ['echo', 'hello'], 'Image' => 'debian:stable')
+      }
+
+      let(:output) { subject.stats }
+      it "returns a Hash" do
+        skip('Not supported on podman') if ::Docker.podman?
+        expect(output).to be_a Hash
+      end
+    end
+
+    context "when streaming container stats" do
+      subject {
+        described_class.create(
+          'Cmd' => ['sleep', '3'],
+          'Image' => 'debian:stable'
+        )
+      }
+
+      it "yields a Hash" do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.start! # If the container isn't started, no stats will be streamed
+        called_count = 0
+        subject.stats do |output|
+          expect(output).to be_a Hash
+          called_count += 1
+          break if called_count == 2
+        end
+        expect(called_count).to eq 2
       end
     end
   end
 
   describe '#logs' do
     subject {
-      described_class.create('Cmd' => "echo hello", 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => ['echo',  'hello'], 'Image' => 'debian:stable')
     }
     after(:each) { subject.remove }
 
     context "when not selecting any stream" do
       let(:non_destination) { subject.logs }
-      it 'raises a client error', :vcr do
+      it 'raises a client error' do
         expect { non_destination }.to raise_error(Docker::Error::ClientError)
       end
     end
 
     context "when selecting stdout" do
       let(:stdout) { subject.logs(stdout: 1) }
-      it 'returns blank logs', :vcr do
+      it 'returns blank logs' do
         expect(stdout).to be_a String
         expect(stdout).to eq ""
       end
@@ -109,7 +149,7 @@ describe Docker::Container do
     subject {
       described_class.create({
         'Cmd' => %w[true],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       }.merge(opts))
     }
 
@@ -117,8 +157,8 @@ describe Docker::Container do
       let(:opts) { {"name" => "bob"} }
       after(:each) { subject.remove }
 
-      it 'should have name set to bob', :vcr do
-        expect(subject.json["Name"]).to eq "/bob"
+      it 'should have name set to bob' do
+        expect(subject.json["Name"]).to eq("/bob")
       end
     end
   end
@@ -128,16 +168,42 @@ describe Docker::Container do
       described_class.create({
         'name' => 'foo',
         'Cmd' => %w[true],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       })
     }
 
     before { subject.start }
-    after(:each) { subject.kill!.remove }
+    after(:each) { subject.tap(&:wait).remove }
 
-    it 'renames the container', :vcr do
+    it 'renames the container' do
+      skip('Not supported on podman') if ::Docker.podman?
       subject.rename('bar')
-      expect(subject.json["Name"]).to eq "bar"
+      expect(subject.json["Name"]).to match(%r{bar})
+    end
+  end
+
+  describe "#update", :docker_1_10 do
+    subject {
+      described_class.create({
+        "name" => "foo",
+        'Cmd' => %w[true],
+        "Image" => "debian:stable",
+        "HostConfig" => {
+          "CpuShares" => 60000
+        }
+      })
+    }
+
+    before { subject.tap(&:start).tap(&:wait) }
+    after(:each) { subject.tap(&:wait).remove }
+
+    it "updates the container" do
+      skip('Podman containers are immutable once created') if ::Docker.podman?
+      subject.refresh!
+      expect(subject.info.fetch("HostConfig").fetch("CpuShares")).to eq 60000
+      subject.update("CpuShares" => 50000)
+      subject.refresh!
+      expect(subject.info.fetch("HostConfig").fetch("CpuShares")).to eq 50000
     end
   end
 
@@ -145,7 +211,7 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => %w[rm -rf /root],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       )
     }
     let(:changes) { subject.changes }
@@ -153,13 +219,14 @@ describe Docker::Container do
     before { subject.tap(&:start).tap(&:wait) }
     after(:each) { subject.tap(&:wait).remove }
 
-    it 'returns the changes as an array', :vcr do
-      expect(changes).to eq [
+    it 'returns the changes as an array' do
+      expect(changes).to be_a(Array)
+      expect(changes).to include(
         {
           "Path" => "/root",
           "Kind" => 2
         },
-      ]
+      )
     end
   end
 
@@ -168,60 +235,178 @@ describe Docker::Container do
       File.join(File.dirname(__FILE__), '..', 'fixtures', 'top')
     }
     let(:image) { Docker::Image.build_from_dir(dir) }
-    let(:top) { sleep 1; container.top }
+    let(:top_empty) { sleep 1; container.top }
+    let(:top_ary) { sleep 1; container.top }
+    let(:top_hash) { sleep 1; container.top(format: :hash) }
     let!(:container) { image.run('/while') }
     after do
       container.kill!.remove
       image.remove
     end
 
-    it 'returns the top commands as an Array', :vcr do
-      expect(top).to be_a Array
-      expect(top).to_not be_empty
-      expect(top.first.keys).to include('PID')
+    it 'returns the top commands as an Array' do
+      expect(top_ary).to be_a Array
+      expect(top_ary).to_not be_empty
+      expect(top_ary.first.keys).to include(/PID/)
+    end
+
+    it 'returns the top commands as an Hash' do
+      expect(top_hash).to be_a Hash
+      expect(top_hash).to_not be_empty
+      expect(top_hash.keys).to eq ['Processes', 'Titles']
+    end
+
+    it 'returns nothing when Processes were not returned due to an error' do
+      expect(Docker::Util).to receive(:parse_json).and_return({}).at_least(:once)
+      expect(top_empty).to eq []
     end
   end
 
-  describe '#copy' do
-    let(:image) { Docker::Image.create('fromImage' => 'debian:wheezy') }
-    subject { image.run('touch /test').tap { |c| c.wait } }
+  describe '#archive_in', :docker_1_8 do
+    let(:license_path) { File.absolute_path(File.join(__FILE__, '..', '..', '..', 'LICENSE')) }
+    subject { Docker::Container.create('Image' => 'debian:stable', 'Cmd' => ['/bin/sh']) }
+    let(:committed_image) { subject.commit }
+    let(:ls_container) { committed_image.run('ls /').tap(&:wait) }
+    let(:output) { ls_container.streaming_logs(stdout: true, stderr: true) }
 
-    after(:each) { subject.remove }
+    after do
+      subject.remove
+    end
+
+    context 'when the input is a tar' do
+      after do
+        ls_container.remove
+        committed_image.remove
+      end
+
+      it 'file exists in the container' do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.archive_in(license_path, '/', overwrite: false)
+        expect(output).to include('LICENSE')
+      end
+    end
+  end
+
+  describe '#archive_in_stream', :docker_1_8 do
+    let(:tar) { StringIO.new(Docker::Util.create_tar('/lol' => 'TEST')) }
+    subject { Docker::Container.create('Image' => 'debian:stable', 'Cmd' => ['/bin/sh']) }
+    let(:committed_image) { subject.commit }
+    let(:ls_container) { committed_image.run('ls /').tap(&:wait) }
+    let(:output) { ls_container.streaming_logs(stdout: true, stderr: true) }
+
+    after do
+      subject.remove
+    end
+
+    context 'when the input is a tar' do
+      after do
+        ls_container.remove
+        committed_image.remove
+      end
+
+      it 'file exists in the container' do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.archive_in_stream('/', overwrite: false) { tar.read }
+        expect(output).to include('lol')
+      end
+    end
+
+    context 'when the input would overwrite a directory with a file' do
+      let(:tar) { StringIO.new(Docker::Util.create_tar('/etc' => 'TEST')) }
+
+      it 'raises an error' do
+        skip('Not supported on podman') if ::Docker.podman?
+        # Docs say this should return a client error: clearly wrong
+        # https://docs.docker.com/engine/reference/api/docker_remote_api_v1.21/
+        # #extract-an-archive-of-files-or-folders-to-a-directory-in-a-container
+        expect {
+          subject.archive_in_stream('/', overwrite: false) { tar.read }
+        }.to raise_error(Docker::Error::ServerError)
+      end
+    end
+  end
+
+  describe '#archive_out', :docker_1_8 do
+    subject { Docker::Container.create('Image' => 'debian:stable', 'Cmd' => ['touch','/test']) }
+
+    after { subject.remove }
 
     context 'when the file does not exist' do
-      it 'raises an error', :vcr do
-        skip 'Docker no longer returns a 500 when the file does not exist'
-        # expect { subject.copy('/lol/not/a/real/file') { |chunk| puts chunk } }
-        #     .to raise_error
+      it 'raises an error' do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.start
+        subject.wait
+
+        expect { subject.archive_out('/lol') { |chunk| puts chunk } }
+          .to raise_error(Docker::Error::NotFoundError)
       end
     end
 
     context 'when the input is a file' do
-      it 'yields each chunk of the tarred file', :vcr do
+      it 'yields each chunk of the tarred file' do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.start; subject.wait
+
         chunks = []
-        subject.copy('/test') { |chunk| chunks << chunk }
+        subject.archive_out('/test') { |chunk| chunks << chunk }
         chunks = chunks.join("\n")
         expect(chunks).to be_include('test')
       end
     end
 
     context 'when the input is a directory' do
-      it 'yields each chunk of the tarred directory', :vcr do
+      it 'yields each chunk of the tarred directory' do
+        skip('Not supported on podman') if ::Docker.podman?
+        subject.start; subject.wait
+
         chunks = []
-        subject.copy('/etc/logrotate.d') { |chunk| chunks << chunk }
+        subject.archive_out('/etc/logrotate.d') { |chunk| chunks << chunk }
         chunks = chunks.join("\n")
         expect(%w[apt dpkg]).to be_all { |file| chunks.include?(file) }
       end
     end
   end
 
+  describe "#read_file", :docker_1_8 do
+    subject {
+      Docker::Container.create(
+        "Image" => "debian:stable",
+        "Cmd" => ["/bin/bash", "-c", "echo \"Hello world\" > /test"]
+      )
+    }
+
+    after { subject.remove }
+
+    before do
+      subject.start
+      subject.wait
+    end
+
+    it "reads contents from files" do
+      skip('Not supported on podman') if ::Docker.podman?
+      expect(subject.read_file("/test")).to eq "Hello world\n"
+    end
+  end
+
+  describe "#store_file", :docker_1_8 do
+    subject { Docker::Container.create('Image' => 'debian:stable', 'Cmd' => ["ls"]) }
+
+    after { subject.remove }
+
+    it "stores content in files" do
+      skip('Not supported on podman') if ::Docker.podman?
+      subject.store_file("/test", "Hello\nWorld")
+      expect(subject.read_file("/test")).to eq "Hello\nWorld"
+    end
+  end
+
   describe '#export' do
-    subject { described_class.create('Cmd' => %w[rm -rf / --no-preserve-root],
+    subject { described_class.create('Cmd' => %w[/true],
                                      'Image' => 'tianon/true') }
     before { subject.start }
     after { subject.tap(&:wait).remove }
 
-    it 'yields each chunk', :vcr do
+    it 'yields each chunk' do
       first = nil
       subject.export do |chunk|
         first ||= chunk
@@ -234,7 +419,7 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => ['bash','-c','sleep 2; echo hello'],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       )
     }
 
@@ -242,7 +427,7 @@ describe Docker::Container do
     after(:each) { subject.stop.remove }
 
     context 'with normal sized chunks' do
-      it 'yields each chunk', :vcr do
+      it 'yields each chunk' do
         chunk = nil
         subject.attach do |stream, c|
           chunk ||= c
@@ -260,7 +445,7 @@ describe Docker::Container do
         Docker.options = {}
       end
 
-      it 'yields each chunk', :vcr do
+      it 'yields each chunk' do
         chunk = nil
         subject.attach do |stream, c|
           chunk ||= c
@@ -271,21 +456,22 @@ describe Docker::Container do
   end
 
   describe '#attach with stdin' do
-    # Because this uses HTTP socket hijacking, it is not compatible with
-    # VCR, so it is currently pending until a good way to test it without
-    # a running Docker daemon is discovered
     it 'yields the output' do
-      skip 'HTTP socket hijacking not compatible with VCR'
+      skip('Currently broken in podman') if ::Docker.podman?
       container = described_class.create(
         'Cmd'       => %w[cat],
-        'Image'     => 'debian:wheezy',
+        'Image'     => 'debian:stable',
         'OpenStdin' => true,
         'StdinOnce' => true
       )
       chunk = nil
-      container.attach(stdin: StringIO.new("foo\nbar\n")) do |stream, c|
-        chunk ||= c
-      end
+      container
+        .tap(&:start)
+        .attach(stdin: StringIO.new("foo\nbar\n")) do |stream, c|
+          chunk ||= c
+        end
+      container.tap(&:wait).remove
+
       expect(chunk).to eq("foo\nbar\n")
     end
   end
@@ -294,16 +480,17 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => %w[test -d /foo],
-        'Image' => 'debian:wheezy',
-        'Volumes' => {'/foo' => {}}
+        'Image' => 'debian:stable',
+        'Volumes' => {'/foo' => {}},
+        'HostConfig' => { 'Binds' => ["/tmp:/foo"] }
       )
     }
-    let(:all) { Docker::Container.all }
+    let(:all) { Docker::Container.all(all: true) }
 
-    before { subject.start('Binds' => ["/tmp:/foo"]) }
+    before { subject.start }
     after(:each) { subject.remove }
 
-    it 'starts the container', :vcr do
+    it 'starts the container' do
       expect(all.map(&:id)).to be_any { |id| id.start_with?(subject.id) }
       expect(subject.wait(10)['StatusCode']).to be_zero
     end
@@ -311,13 +498,13 @@ describe Docker::Container do
 
   describe '#stop' do
     subject {
-      described_class.create('Cmd' => %w[true], 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => %w[true], 'Image' => 'debian:stable')
     }
 
     before { subject.tap(&:start).stop('timeout' => '10') }
     after { subject.remove }
 
-    it 'stops the container', :vcr do
+    it 'stops the container' do
       expect(described_class.all(:all => true).map(&:id)).to be_any { |id|
         id.start_with?(subject.id)
       }
@@ -325,13 +512,49 @@ describe Docker::Container do
         id.start_with?(subject.id)
       }
     end
+
+    context 'with a timeout' do
+      let(:custom_timeout) { 60 }
+
+      before do
+        subject.tap(&:start)
+      end
+
+      it 'extends the Excon timeout ensuring the request does not timeout before Docker' do
+        expect(subject.connection).to receive(:request).with(
+          :post,
+          anything,
+          anything,
+          hash_including(read_timeout: custom_timeout + 5, write_timeout: custom_timeout + 5)
+        ).once
+        allow(subject.connection).to receive(:request).with(:delete, anything, anything)
+        subject.stop('timeout' => custom_timeout)
+      end
+    end
+
+    context 'without a timeout' do
+      before do
+        subject.tap(&:start)
+      end
+
+      it 'does not adjust the default Excon HTTP timeout' do
+        expect(subject.connection).to receive(:request).with(
+          :post,
+          anything,
+          anything,
+          hash_including(body: '{}')
+        ).once
+        allow(subject.connection).to receive(:request).with(:delete, anything, anything)
+        subject.stop
+      end
+    end
   end
 
   describe '#exec' do
     subject {
       described_class.create(
         'Cmd' => %w[sleep 20],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       ).start
     }
     after { subject.kill!.remove }
@@ -339,22 +562,22 @@ describe Docker::Container do
     context 'when passed only a command' do
       let(:output) { subject.exec(['bash','-c','sleep 2; echo hello']) }
 
-      it 'returns the stdout/stderr messages and exit code', :vcr do
+      it 'returns the stdout/stderr messages and exit code' do
         expect(output).to eq([["hello\n"], [], 0])
       end
     end
 
     context 'when detach is true' do
-      let(:output) { subject.exec('date', detach: true) }
+      let(:output) { subject.exec(['date'], detach: true) }
 
-      it 'returns the Docker::Exec object', :vcr do
+      it 'returns the Docker::Exec object' do
         expect(output).to be_a Docker::Exec
         expect(output.id).to_not be_nil
       end
     end
 
     context 'when passed a block' do
-      it 'streams the stdout/stderr messages', :vcr do
+      it 'streams the stdout/stderr messages' do
         chunk = nil
         subject.exec(['bash','-c','sleep 2; echo hello']) do |stream, c|
           chunk ||= c
@@ -364,10 +587,10 @@ describe Docker::Container do
     end
 
     context 'when stdin object is passed' do
-      let(:output) { subject.exec('cat', stdin: StringIO.new("hello")) }
+      let(:output) { subject.exec(['cat'], stdin: StringIO.new("hello")) }
 
-      it 'returns the stdout/stderr messages', :vcr do
-        skip 'HTTP socket hijacking not compatible with VCR'
+      it 'returns the stdout/stderr messages' do
+        skip('Not supported on podman') if ::Docker.podman?
         expect(output).to eq([["hello"],[],0])
       end
     end
@@ -379,7 +602,7 @@ describe Docker::Container do
       ] }
       let(:output) { subject.exec(command, tty: true) }
 
-      it 'returns the raw stdout/stderr output', :vcr do
+      it 'returns the raw stdout/stderr output' do
         expect(output).to eq([["I'm a TTY!"], [], 0])
       end
     end
@@ -388,13 +611,13 @@ describe Docker::Container do
   describe '#kill' do
     let(:command) { ['/bin/bash', '-c', 'while [ 1 ]; do echo hello; done'] }
     subject {
-      described_class.create('Cmd' => command, 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => command, 'Image' => 'debian:stable')
     }
 
     before { subject.start }
     after(:each) {subject.remove }
 
-    it 'kills the container', :vcr do
+    it 'kills the container' do
       subject.kill
       expect(described_class.all.map(&:id)).to be_none { |id|
         id.start_with?(subject.id)
@@ -412,7 +635,7 @@ describe Docker::Container do
           'trap echo SIGTERM; while [ 1 ]; do echo hello; done'
         ]
       }
-      it 'kills the container', :vcr do
+      it 'kills the container' do
         subject.kill(:signal => "SIGTERM")
         expect(described_class.all.map(&:id)).to be_any { |id|
           id.start_with?(subject.id)
@@ -434,10 +657,10 @@ describe Docker::Container do
 
   describe '#delete' do
     subject {
-      described_class.create('Cmd' => ['ls'], 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => ['ls'], 'Image' => 'debian:stable')
     }
 
-    it 'deletes the container', :vcr do
+    it 'deletes the container' do
       subject.delete(:force => true)
       expect(described_class.all.map(&:id)).to be_none { |id|
         id.start_with?(subject.id)
@@ -447,13 +670,13 @@ describe Docker::Container do
 
   describe '#restart' do
     subject {
-      described_class.create('Cmd' => %w[sleep 10], 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => %w[sleep 10], 'Image' => 'debian:stable')
     }
 
     before { subject.start }
     after { subject.kill!.remove }
 
-    it 'restarts the container', :vcr do
+    it 'restarts the container' do
       expect(described_class.all.map(&:id)).to be_any { |id|
         id.start_with?(subject.id)
       }
@@ -472,12 +695,13 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => %w[sleep 50],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       ).start
     }
     after { subject.unpause.kill!.remove }
 
-    it 'pauses the container', :vcr do
+    it 'pauses the container' do
+      skip('Not supported on rootless podman') if (::Docker.podman? && ::Docker.rootless?)
       subject.pause
       expect(described_class.get(subject.id).info['State']['Paused']).to be true
     end
@@ -487,13 +711,13 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => %w[sleep 50],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       ).start
     }
     before { subject.pause }
     after { subject.kill!.remove }
 
-    it 'unpauses the container', :vcr do
+    it 'unpauses the container' do
       subject.unpause
       expect(
         described_class.get(subject.id).info['State']['Paused']
@@ -505,33 +729,31 @@ describe Docker::Container do
     subject {
       described_class.create(
         'Cmd' => %w[tar nonsense],
-        'Image' => 'debian:wheezy'
+        'Image' => 'debian:stable'
       )
     }
 
     before { subject.start }
     after(:each) { subject.remove }
 
-    it 'waits for the command to finish', :vcr do
+    it 'waits for the command to finish' do
       expect(subject.wait['StatusCode']).to_not be_zero
     end
 
     context 'when an argument is given' do
       subject { described_class.create('Cmd' => %w[sleep 5],
-                                       'Image' => 'debian:wheezy') }
+                                       'Image' => 'debian:stable') }
 
-      it 'sets the :read_timeout to that amount of time', :vcr do
+      it 'sets the :read_timeout to that amount of time' do
         expect(subject.wait(6)['StatusCode']).to be_zero
       end
 
-      # context 'and a command runs for too long' do
-      #   after(:each) { subject.remove }
-      #
-      #   it 'raises a ServerError', :vcr do
-      #     skip "VCR doesn't like to record errors"
-      #     expect{subject.wait(4)}.to raise_error(Docker::Error::TimeoutError)
-      #   end
-      # end
+      context 'and a command runs for too long' do
+        it 'raises a ServerError' do
+          expect{subject.wait(4)}.to raise_error(Docker::Error::TimeoutError)
+          subject.tap(&:wait)
+        end
+      end
     end
   end
 
@@ -540,13 +762,13 @@ describe Docker::Container do
 
     context 'when the Container\'s command does not return status code of 0' do
       subject { described_class.create('Cmd' => %w[false],
-                                       'Image' => 'debian:wheezy') }
+                                       'Image' => 'debian:stable') }
 
       after do
         subject.remove
       end
 
-      it 'raises an error', :vcr do
+      it 'raises an error' do
         expect { run_command }
             .to raise_error(Docker::Error::UnexpectedResponseError)
       end
@@ -554,7 +776,7 @@ describe Docker::Container do
 
     context 'when the Container\'s command returns a status code of 0' do
       subject { described_class.create('Cmd' => %w[pwd],
-                                       'Image' => 'debian:wheezy') }
+                                       'Image' => 'debian:stable') }
       after do
         subject.remove
         image = run_command.json['Image']
@@ -565,7 +787,7 @@ describe Docker::Container do
         end
       end
 
-      it 'creates a new container to run the specified command', :vcr do
+      it 'creates a new container to run the specified command' do
         expect(run_command.wait['StatusCode']).to be_zero
       end
     end
@@ -573,7 +795,7 @@ describe Docker::Container do
 
   describe '#commit' do
     subject {
-      described_class.create('Cmd' => %w[true], 'Image' => 'debian:wheezy')
+      described_class.create('Cmd' => %w[true], 'Image' => 'debian:stable')
     }
     let(:image) { subject.commit }
 
@@ -582,22 +804,24 @@ describe Docker::Container do
       image.remove
     end
 
-    it 'creates a new Image from the  Container\'s changes', :vcr do
+    it 'creates a new Image from the  Container\'s changes' do
       subject.tap(&:start).wait
 
       expect(image).to be_a Docker::Image
       expect(image.id).to_not be_nil
     end
 
-    # context 'if run is passed, it saves the command in the image', :vcr do
-    #   let(:image) { subject.commit }
-    #
-    #   it 'saves the command' do
-    #     skip 'This is no longer working in v0.8'
-    #     expect(image.run('pwd').attach).to eql [["/\n"],[]]
-    #   end
-    # end
+    context 'if run is passed, it saves the command in the image' do
+      let(:image) { subject.commit }
+      let(:container) { image.run('pwd') }
 
+      it 'saves the command' do
+        skip('Not supported on podman') if ::Docker.podman?
+        container.wait
+        expect(container.attach(logs: true, stream: false)).to eql [["/\n"],[]]
+        container.remove
+      end
+    end
   end
 
   describe '.create' do
@@ -622,29 +846,14 @@ describe Docker::Container do
       context 'when the HTTP request returns a 200' do
         let(:options) do
           {
-            "Hostname"     => "",
-            "User"         => "",
-            "Memory"       => 0,
-            "MemorySwap"   => 0,
-            "AttachStdin"  => false,
-            "AttachStdout" => false,
-            "AttachStderr" => false,
-            "PortSpecs"    => nil,
-            "Tty"          => false,
-            "OpenStdin"    => false,
-            "StdinOnce"    => false,
-            "Env"          => nil,
             "Cmd"          => ["date"],
-            "Dns"          => nil,
-            "Image"        => "debian:wheezy",
-            "Volumes"      => {},
-            "VolumesFrom"  => ""
+            "Image"        => "debian:stable",
           }
         end
         let(:container) { subject.create(options) }
         after { container.remove }
 
-        it 'sets the id', :vcr do
+        it 'sets the id' do
           expect(container).to be_a Docker::Container
           expect(container.id).to_not be_nil
           expect(container.connection).to_not be_nil
@@ -674,11 +883,11 @@ describe Docker::Container do
 
     context 'when the HTTP response is a 200' do
       let(:container) {
-        subject.create('Cmd' => ['ls'], 'Image' => 'debian:wheezy')
+        subject.create('Cmd' => ['ls'], 'Image' => 'debian:stable')
       }
       after { container.remove }
 
-      it 'materializes the Container into a Docker::Container', :vcr do
+      it 'materializes the Container into a Docker::Container' do
         expect(subject.get(container.id)).to be_a Docker::Container
       end
     end
@@ -706,12 +915,12 @@ describe Docker::Container do
 
     context 'when the HTTP response is a 200' do
       let(:container) {
-        subject.create('Cmd' => ['ls'], 'Image' => 'debian:wheezy')
+        subject.create('Cmd' => ['ls'], 'Image' => 'debian:stable')
       }
       before { container }
       after { container.remove }
 
-      it 'materializes each Container into a Docker::Container', :vcr do
+      it 'materializes each Container into a Docker::Container' do
         expect(subject.all(:all => true)).to be_all { |container|
           container.is_a?(Docker::Container)
         }
@@ -719,4 +928,10 @@ describe Docker::Container do
       end
     end
   end
+
+  describe '.prune', :docker_17_03 => true do
+    it 'prune containers' do
+      expect { Docker::Container.prune }.not_to raise_error
+    end
+  end
 end
