@@ -1,5 +1,7 @@
 require 'spec_helper'
 
+SingleCov.covered! uncovered: 16
+
 describe Docker::Image do
   describe '#to_s' do
     subject { described_class.new(Docker.connection, info) }
@@ -8,7 +10,7 @@ describe Docker::Image do
     let(:connection) { Docker.connection }
 
     let(:info) do
-      {"id" => "bf119e2", "Repository" => "debian", "Tag" => "wheezy",
+      {"id" => "bf119e2", "Repository" => "debian", "Tag" => "stable",
         "Created" => 1364102658, "Size" => 24653, "VirtualSize" => 180116135}
     end
 
@@ -21,17 +23,28 @@ describe Docker::Image do
   end
 
   describe '#remove' do
-
     context 'when no name is given' do
       let(:id) { subject.id }
-      subject { described_class.create('fromImage' => 'busybox') }
+      subject { described_class.create('fromImage' => 'busybox:latest') }
+      after { described_class.create('fromImage' => 'busybox:latest') }
 
-      it 'removes the Image', :vcr do
+      it 'removes the Image' do
         subject.remove(:force => true)
         expect(Docker::Image.all.map(&:id)).to_not include(id)
       end
     end
 
+    context 'when using the class' do
+      let(:id) { subject.id }
+      subject { described_class.create('fromImage' => 'busybox:latest') }
+      after { described_class.create('fromImage' => 'busybox:latest') }
+
+      it 'removes the Image' do
+        Docker::Image.remove(id, force: true)
+        expect(Docker::Image.all.map(&:id)).to_not include(id)
+      end
+    end
+
     context 'when a valid tag is given' do
       it 'untags the Image'
     end
@@ -44,7 +57,7 @@ describe Docker::Image do
   describe '#insert_local' do
     include_context "local paths"
 
-    subject { described_class.create('fromImage' => 'debian:wheezy') }
+    subject { described_class.create('fromImage' => 'debian:stable') }
 
     let(:rm) { false }
     let(:new_image) {
@@ -56,7 +69,7 @@ describe Docker::Image do
     context 'when the local file does not exist' do
       let(:file) { '/lol/not/a/file' }
 
-      it 'raises an error', :vcr do
+      it 'raises an error' do
         expect { new_image }.to raise_error(Docker::Error::ArgumentError)
       end
     end
@@ -64,19 +77,23 @@ describe Docker::Image do
     context 'when the local file does exist' do
       let(:file) { File.join(project_dir, 'Gemfile') }
       let(:gemfile) { File.read('Gemfile') }
-      let(:container) { new_image.run('cat /Gemfile') }
+      let(:container) { new_image.run('cat /Gemfile').tap(&:wait) }
       after do
-        container.tap(&:wait).remove
+        container.remove
         new_image.remove
       end
 
-      it 'creates a new Image that has that file', :vcr do
-        output = container.streaming_logs(stdout: true)
-        expect(output).to eq(gemfile)
+      it 'creates a new Image that has that file' do
+        begin
+          output = container.streaming_logs(stdout: true)
+          expect(output).to eq(gemfile)
+        rescue Docker::Error::UnexpectedResponseError => ex
+          skip("Could not communicate with DockerHub: #{ex}")
+        end
       end
     end
 
-    context 'when a direcory is passed' do
+    context 'when a directory is passed' do
       let(:new_image) {
         subject.insert_local(
           'localPath' => File.join(project_dir, 'lib'),
@@ -90,8 +107,12 @@ describe Docker::Image do
         new_image.remove
       end
 
-      it 'inserts the directory', :vcr do
-        expect(response.split("\n").sort).to eq(Dir.entries('lib/docker').sort)
+      it 'inserts the directory' do
+        begin
+          expect(response.split("\n").sort).to eq(Dir.entries('lib/docker').sort)
+        rescue Docker::Error::UnexpectedResponseError => ex
+          skip("Could not communicate with DockerHub: #{ex}")
+        end
       end
     end
 
@@ -103,15 +124,19 @@ describe Docker::Image do
       let(:license) { File.read('LICENSE') }
       let(:container) { new_image.run('cat /Gemfile /LICENSE') }
       let(:response) {
-        container.streaming_logs(stdout: true)
+        container.tap(&:wait).streaming_logs(stdout: true)
       }
       after do
-        container.tap(&:wait).remove
+        container.remove
         new_image.remove
       end
 
-      it 'creates a new Image that has each file', :vcr do
-        expect(response).to eq("#{gemfile}#{license}")
+      it 'creates a new Image that has each file' do
+        begin
+          expect(response).to eq("#{gemfile}#{license}")
+        rescue Docker::Error::UnexpectedResponseError => ex
+          skip("Could not communicate with DockerHub: #{ex}")
+        end
       end
     end
 
@@ -120,14 +145,22 @@ describe Docker::Image do
       let(:file) { File.join(project_dir, 'Gemfile') }
       after(:each) { new_image.remove }
 
-      it 'leave no intermediate containers', :vcr do
-        expect { new_image }.to change {
-          Docker::Container.all(:all => true).count
-        }.by 0
+      it 'leave no intermediate containers' do
+        begin
+          expect { new_image }.to change {
+            Docker::Container.all(:all => true).count
+          }.by 0
+        rescue Docker::Error::UnexpectedResponseError => ex
+          skip("Could not communicate with DockerHub: #{ex}")
+        end
       end
 
-      it 'creates a new image', :vcr do
-        expect{new_image}.to change{Docker::Image.all.count}.by 1
+      it 'creates a new image' do
+        begin
+          expect{new_image}.to change{Docker::Image.all.count}.by 1
+        rescue Docker::Error::UnexpectedResponseError => ex
+          skip("Could not communicate with DockerHub: #{ex}")
+        end
       end
     end
   end
@@ -147,13 +180,15 @@ describe Docker::Image do
     }
     after { image.remove(:name => repo_tag, :noprune => true) }
 
-    it 'pushes the Image', :vcr do
+    it 'pushes the Image' do
+      skip_without_auth
       image.push(credentials)
     end
 
-    it 'streams output from push', :vcr do
+    it 'streams output from push' do
+      skip_without_auth
       expect { |b| image.push(credentials, &b) }
-        .to yield_control
+        .to yield_control.at_least(1)
     end
 
     context 'when a tag is specified' do
@@ -167,7 +202,8 @@ describe Docker::Image do
       }
 
       context 'when no tag is specified' do
-        it 'looks up the first repo tag', :vcr do
+        it 'looks up the first repo tag' do
+          skip_without_auth
           expect { image.push }.to_not raise_error
         end
       end
@@ -177,37 +213,47 @@ describe Docker::Image do
       let(:credentials) { nil }
       let(:repo_tag) { "localhost:5000/true" }
 
-      it 'still pushes', :vcr do
-        expect { image.push }.to_not raise_error
+      it 'still pushes' do
+        begin
+          image.push
+        rescue => ex
+          if ex.message =~ /connection refused/
+            skip("Registry at #{repo_tag} is not available")
+          else
+            expect { raise(ex) }.to_not raise_error
+          end
+        end
       end
     end
   end
 
   describe '#tag' do
-    subject { described_class.create('fromImage' => 'debian:wheezy') }
+    subject { described_class.create('fromImage' => 'debian:stable') }
     after { subject.remove(:name => 'teh:latest', :noprune => true) }
 
-    it 'tags the image with the repo name', :vcr do
+    it 'tags the image with the repo name' do
       subject.tag(:repo => :teh, :force => true)
       expect(subject.info['RepoTags']).to include 'teh:latest'
     end
   end
 
   describe '#json' do
-    subject { described_class.create('fromImage' => 'debian:wheezy') }
+    before { skip_without_auth }
+
+    subject { described_class.create('fromImage' => 'debian:stable') }
     let(:json) { subject.json }
 
-    it 'returns additional information about image image', :vcr do
+    it 'returns additional information about image image' do
       expect(json).to be_a Hash
       expect(json.length).to_not be_zero
     end
   end
 
   describe '#history' do
-    subject { described_class.create('fromImage' => 'debian:wheezy') }
+    subject { described_class.create('fromImage' => 'debian:stable') }
     let(:history) { subject.history }
 
-    it 'returns the history of the Image', :vcr do
+    it 'returns the history of the Image' do
       expect(history).to be_a Array
       expect(history.length).to_not be_zero
       expect(history).to be_all { |elem| elem.is_a? Hash }
@@ -215,47 +261,67 @@ describe Docker::Image do
   end
 
   describe '#run' do
-    subject { described_class.create('fromImage' => 'debian:wheezy') }
-    let(:container) { subject.run(cmd) }
+    let(:cmd) { nil }
+    let(:options) { {} }
+
+    subject do
+      described_class.create(
+        {'fromImage' => 'debian:stable'})
+    end
+
+    let(:container) { subject.run(cmd, options).tap(&:wait) }
     let(:output) { container.streaming_logs(stdout: true) }
 
-    context 'when the argument is a String', :vcr do
+    context 'when cmd is a String' do
       let(:cmd) { 'ls /lib64/' }
       after { container.remove }
 
       it 'splits the String by spaces and creates a new Container' do
-        container.wait
         expect(output).to eq("ld-linux-x86-64.so.2\n")
       end
     end
 
-    context 'when the argument is an Array' do
+    context 'when cmd is an Array' do
       let(:cmd) { %w[which pwd] }
-      after { container.tap(&:wait).remove }
+      after { container.remove }
 
-      it 'creates a new Container', :vcr do
+      it 'creates a new Container' do
         expect(output).to eq("/bin/pwd\n")
       end
     end
 
-    context 'when the argument is nil', :vcr  do
+    context 'when cmd is nil', docker_1_12: true do
       let(:cmd) { nil }
+
       context 'no command configured in image' do
-        subject { described_class.create('fromImage' => 'scratch') }
+        subject { described_class.create('fromImage' => 'swipely/base') }
         it 'should raise an error if no command is specified' do
-          expect {container}.to raise_error(Docker::Error::ServerError,
-                                         "No command specified.")
+          begin
+            container
+          rescue => ex
+            expect([Docker::Error::ServerError, Docker::Error::ClientError]).to include(ex.class)
+            expect(ex.message).to match(/No\ command\ specified/)
+          end
         end
       end
+    end
 
-      context "command configured in image" do
-        let(:cmd) { 'pwd' }
-        after { container.remove }
+    context "command configured in image" do
+      let(:cmd) { 'pwd' }
+      after { container.remove }
 
-        it 'should normally show result if image has Cmd configured' do
-          container.wait
-          expect(output).to eql "/\n"
-        end
+      it 'should normally show result if image has Cmd configured' do
+        expect(output).to eql "/\n"
+      end
+    end
+
+    context 'when using cpu shares' do
+      let(:options) { { 'CpuShares' => 50 } }
+      after { container.remove }
+
+      it 'returns 50' do
+        skip('Not supported on podman') if ::Docker.podman?
+        expect(container.json["HostConfig"]["CpuShares"]).to eq 50
       end
     end
   end
@@ -263,17 +329,28 @@ describe Docker::Image do
   describe '#save' do
     let(:image) { Docker::Image.get('busybox') }
 
-    it 'calls the class method', :vcr do
+    it 'calls the class method' do
       expect(Docker::Image).to receive(:save)
         .with(image.id, 'busybox.tar', anything)
       image.save('busybox.tar')
     end
   end
 
-  describe '#refresh!' do
-    let(:image) { Docker::Image.create('fromImage' => 'debian:wheezy') }
+  describe '#save_stream' do
+    let(:image) { Docker::Image.get('busybox') }
+    let(:block) { proc { |chunk| puts chunk } }
 
-    it 'updates the @info hash', :vcr do
+    it 'calls the class method' do
+      expect(Docker::Image).to receive(:save_stream)
+        .with(image.id, instance_of(Hash), instance_of(Docker::Connection))
+      image.save_stream(:chunk_size => 1024 * 1024, &block)
+    end
+  end
+
+  describe '#refresh!' do
+    let(:image) { Docker::Image.create('fromImage' => 'debian:stable') }
+
+    it 'updates the @info hash' do
       size = image.info.size
       image.refresh!
       expect(image.info.size).to be > size
@@ -282,24 +359,43 @@ describe Docker::Image do
     context 'with an explicit connection' do
       let(:connection) { Docker::Connection.new(Docker.url, Docker.options) }
       let(:image) {
-        Docker::Image.create({'fromImage' => 'debian:wheezy'}, nil, connection)
+        Docker::Image.create({'fromImage' => 'debian:stable'}, nil, connection)
       }
 
-      it 'updates using the provided connection', :vcr do
-        expect(connection).to receive(:get)
-          .with('/images/json', all: true).ordered
-        expect(connection).to receive(:get)
-          .with("/images/#{image.id}/json", {}).ordered.and_call_original
+      it 'updates using the provided connection' do
         image.refresh!
       end
     end
   end
 
+  describe '.load' do
+    include_context "local paths"
+    let(:file) { File.join(project_dir, 'spec', 'fixtures', 'load.tar') }
+
+    context 'when the argument is a String' do
+      it 'loads tianon/true image from the file system' do
+        result = Docker::Image.load(file)
+        expect(result).to eq("")
+      end
+    end
+
+    context 'when the argument is an IO' do
+      let(:io) { File.open(file) }
+
+      after { io.close }
+
+      it 'loads tinan/true image from the IO' do
+        result = Docker::Image.load(io)
+        expect(result).to eq("")
+      end
+    end
+  end
+
   describe '.create' do
     subject { described_class }
 
     context 'when the Image does not yet exist and the body is a Hash' do
-      let(:image) { subject.create('fromImage' => 'swipely/scratch') }
+      let(:image) { subject.create('fromImage' => 'swipely/base') }
       let(:creds) {
         {
           :username => ENV['DOCKER_API_USER'],
@@ -308,28 +404,76 @@ describe Docker::Image do
         }
       }
 
-      before { Docker.creds = creds }
-      after { image.remove(:name => 'swipely/scratch', :noprune => true) }
+      before do
+        skip_without_auth
+        Docker::Image.create('fromImage' => 'swipely/base').remove
+      end
+      after { Docker::Image.create('fromImage' => 'swipely/base') }
 
-      it 'sets the id and sends Docker.creds', :vcr do
+      it 'sets the id and sends Docker.creds' do
+        allow(Docker).to receive(:creds).and_return(creds)
         expect(image).to be_a Docker::Image
-        expect(image.id).to match(/\A[a-fA-F0-9]+\Z/)
+        expect(image.id).to match(/\A(sha256:)?[a-fA-F0-9]+\Z/)
         expect(image.id).to_not include('base')
         expect(image.id).to_not be_nil
         expect(image.id).to_not be_empty
-        expect(image.info[:headers].keys).to include 'X-Registry-Auth'
+      end
+    end
+
+    context 'image with tag' do
+      it 'pulls the image (string arguments)' do
+        image = subject.create('fromImage' => 'busybox', 'tag' => 'uclibc')
+        image.refresh!
+        expect(image.info['RepoTags']).to include(/busybox:uclibc$/)
+      end
+
+      it 'pulls the image (symbol arguments)' do
+        image = subject.create(fromImage: 'busybox', tag: 'uclibc')
+        image.refresh!
+        expect(image.info['RepoTags']).to include(/busybox:uclibc$/)
+      end
+
+      it 'supports identical fromImage and tag', docker_1_10: true do
+        # This is here for backwards compatibility. docker-api used to
+        # complete ignore the "tag" argument, which Docker itself prioritizes
+        # over a tag found in fromImage, which meant that we had 3 scenarios:
+        #
+        # 1 fromImage does not include a tag, and the tag argument is provided
+        #   and isn't the default (i.e. "latest"): docker-api crashes looking
+        #   for fromImage when the image that was pulled is fromImage:tag (or
+        #   returns the wrong image if fromImage:latest exists)
+        # 2 fromImage does not a include a tag, and the tag argument is absent
+        #   or default (i.e. "latest"): docker-api finds the right image.
+        # 3 fromImage includes a tag, and the tag argument is absent: docker-api
+        #   also finds the right image.
+        # 4 fromImage includes a tag, and the tag argument is present: works if
+        #   the tag is the same in both.
+        #
+        # Adding support for the tag argument to fix 1 above means we'd break 4
+        # if we didn't explicitly handle the case where both tags are identical.
+        # This is what this test checks.
+        #
+        # Note that providing the tag inline in fromImage is only supported in
+        # Docker 1.10 and up.
+        skip('Not supported on podman') if ::Docker.podman?
+        image = subject.create(fromImage: 'busybox:uclibc', tag: 'uclibc')
+        image.refresh!
+        expect(image.info['RepoTags']).to include('busybox:uclibc')
       end
     end
 
     context 'with a block capturing create output' do
       let(:create_output) { "" }
       let(:block) { Proc.new { |chunk| create_output << chunk } }
-      let!(:image) { subject.create('fromImage' => 'tianon/true', &block) }
 
-      before { Docker.creds = nil }
+      before do
+        Docker.creds = nil
+        subject.create('fromImage' => 'busybox').remove(force: true)
+      end
 
-      it 'calls the block and passes build output', :vcr do
-        expect(create_output).to match(/Pulling repository tianon\/true/)
+      it 'calls the block and passes build output' do
+        subject.create('fromImage' => 'busybox', &block)
+        expect(create_output).to match(/ulling.*busybox/)
       end
     end
   end
@@ -339,9 +483,9 @@ describe Docker::Image do
     let(:image) { subject.get(image_name) }
 
     context 'when the image does exist' do
-      let(:image_name) { 'debian:wheezy' }
+      let(:image_name) { 'debian:stable' }
 
-      it 'returns the new image', :vcr do
+      it 'returns the new image' do
         expect(image).to be_a Docker::Image
       end
     end
@@ -359,7 +503,7 @@ describe Docker::Image do
         Excon.stubs.shift
       end
 
-      it 'raises a not found error', :vcr do
+      it 'raises a not found error' do
         expect { image }.to raise_error(Docker::Error::NotFoundError)
       end
     end
@@ -372,29 +516,50 @@ describe Docker::Image do
       let(:file) { "#{project_dir}/scratch.tar" }
       after { FileUtils.remove(file) }
 
-      it 'exports tarball of image to specified file', :vcr do
-        Docker::Image.save('scratch', file)
+      it 'exports tarball of image to specified file' do
+        Docker::Image.save('swipely/base', file)
         expect(File.exist?(file)).to eq true
         expect(File.read(file)).to_not be_nil
       end
     end
 
     context 'when no filename is specified' do
-      it 'returns raw binary data as string', :vcr do
-        raw = Docker::Image.save('scratch:latest')
+      it 'returns raw binary data as string' do
+        raw = Docker::Image.save('swipely/base')
         expect(raw).to_not be_nil
       end
     end
   end
 
+  describe '.save_stream' do
+    let(:image) { 'busybox:latest' }
+    let(:non_streamed) do
+      Docker.connection.get('/images/get', 'names' => image)
+    end
+    let(:streamed) { '' }
+    let(:tar_files) do
+      proc do |string|
+        Gem::Package::TarReader
+          .new(StringIO.new(string, 'rb'))
+          .map(&:full_name)
+          .sort
+      end
+    end
+
+    it 'yields each chunk of the image' do
+      Docker::Image.save_stream(image) { |chunk| streamed << chunk }
+      expect(tar_files.call(streamed)).to eq(tar_files.call(non_streamed))
+    end
+  end
+
   describe '.exist?' do
     subject { described_class }
     let(:exists) { subject.exist?(image_name) }
 
     context 'when the image does exist' do
-      let(:image_name) { 'debian:wheezy' }
+      let(:image_name) { 'debian:stable' }
 
-      it 'returns true', :vcr do
+      it 'returns true' do
         expect(exists).to eq(true)
       end
     end
@@ -412,7 +577,7 @@ describe Docker::Image do
         Excon.stubs.shift
       end
 
-      it 'return false', :vcr do
+      it 'return false' do
         expect(exists).to eq(false)
       end
     end
@@ -437,7 +602,7 @@ describe Docker::Image do
       let(:import) { subject.import(file) }
       after { import.remove(:noprune => true) }
 
-      it 'creates the Image', :vcr do
+      it 'creates the Image' do
         expect(import).to be_a Docker::Image
         expect(import.id).to_not be_nil
       end
@@ -445,7 +610,7 @@ describe Docker::Image do
 
     context 'when the argument is a URI' do
       context 'when the URI is invalid' do
-        it 'raises an error', :vcr do
+        it 'raises an error' do
           expect { subject.import('http://google.com') }
             .to raise_error(Docker::Error::IOError)
         end
@@ -456,7 +621,7 @@ describe Docker::Image do
         let(:import) { subject.import(uri) }
         after { import.remove(:noprune => true) }
 
-        it 'returns an Image', :vcr do
+        it 'returns an Image' do
           expect(import).to be_a Docker::Image
           expect(import.id).to_not be_nil
         end
@@ -468,9 +633,9 @@ describe Docker::Image do
     subject { described_class }
 
     let(:images) { subject.all(:all => true) }
-    before { subject.create('fromImage' => 'debian:wheezy') }
+    before { subject.create('fromImage' => 'debian:stable') }
 
-    it 'materializes each Image into a Docker::Image', :vcr do
+    it 'materializes each Image into a Docker::Image' do
       images.each do |image|
         expect(image).to_not be_nil
 
@@ -478,7 +643,14 @@ describe Docker::Image do
 
         expect(image.id).to_not be_nil
 
-        %w(Created Size VirtualSize).each do |key|
+        expected = [
+          'Created',
+          'Size'
+        ]
+
+        expected << 'VirtualSize' unless ::Docker.podman?
+
+        expected.each do |key|
           expect(image.info).to have_key(key)
         end
       end
@@ -487,30 +659,50 @@ describe Docker::Image do
     end
   end
 
-  describe '.search' do
-    subject { described_class }
+  describe '.prune', :docker_17_03 => true do
+    it 'prune images' do
+      expect { Docker::Image.prune }.not_to raise_error
+    end
+  end
 
-    it 'materializes each Image into a Docker::Image', :vcr do
-      expect(subject.search('term' => 'sshd')).to be_all { |image|
-        !image.id.nil? && image.is_a?(described_class)
-      }
+  unless ::Docker.podman?
+    describe '.search' do
+      subject { described_class }
+
+      it 'materializes each Image into a Docker::Image' do
+        expect(subject.search('term' => 'sshd')).to be_all { |image|
+          !image.id.nil? && image.is_a?(described_class)
+        }
+      end
     end
   end
 
   describe '.build' do
     subject { described_class }
     context 'with an invalid Dockerfile' do
-      it 'throws a UnexpectedResponseError', :vcr do
-        expect { subject.build('lololol') }
-            .to raise_error(Docker::Error::UnexpectedResponseError)
+      if ::Docker.podman?
+        it 'throws a UnexpectedResponseError' do
+          expect { subject.build('lololol') }
+              .to raise_error(Docker::Error::UnexpectedResponseError)
+        end
+      else
+        it 'throws a UnexpectedResponseError', docker_17_09: false do
+          expect { subject.build('lololol') }
+              .to raise_error(Docker::Error::ClientError)
+        end
+
+        it 'throws a ClientError', docker_17_09: true do
+            expect { subject.build('lololol') }
+                .to raise_error(Docker::Error::ClientError)
+        end
       end
     end
 
     context 'with a valid Dockerfile' do
       context 'without query parameters' do
-        let(:image) { subject.build("FROM debian:wheezy\n") }
+        let(:image) { subject.build("FROM debian:stable\n") }
 
-        it 'builds an image', :vcr do
+        it 'builds an image' do
           expect(image).to be_a Docker::Image
           expect(image.id).to_not be_nil
           expect(image.connection).to be_a Docker::Connection
@@ -520,30 +712,29 @@ describe Docker::Image do
       context 'with specifying a repo in the query parameters' do
         let(:image) {
           subject.build(
-            "FROM debian:wheezy\nRUN true\n",
+            "FROM debian:stable\nRUN true\n",
             "t" => "#{ENV['DOCKER_API_USER']}/debian:true"
           )
         }
         after { image.remove(:noprune => true) }
 
-        it 'builds an image and tags it', :vcr do
+        it 'builds an image and tags it' do
           expect(image).to be_a Docker::Image
           expect(image.id).to_not be_nil
           expect(image.connection).to be_a Docker::Connection
           image.refresh!
-          expect(image.info["RepoTags"]).to eq(
-            ["#{ENV['DOCKER_API_USER']}/debian:true"]
-          )
+          expect(image.info["RepoTags"].size).to eq(1)
+          expect(image.info["RepoTags"].first).to match(%r{#{ENV['DOCKER_API_USER']}/debian:true})
         end
       end
 
       context 'with a block capturing build output' do
         let(:build_output) { "" }
         let(:block) { Proc.new { |chunk| build_output << chunk } }
-        let!(:image) { subject.build("FROM debian:wheezy\n", &block) }
+        let!(:image) { subject.build("FROM debian:stable\n", &block) }
 
-        it 'calls the block and passes build output', :vcr do
-          expect(build_output).to match(/Step 0 : FROM debian:wheezy/)
+        it 'calls the block and passes build output' do
+          expect(build_output).to match(/(Step|STEP) \d(\/\d)?\s?: FROM debian:stable/)
         end
       end
     end
@@ -561,39 +752,31 @@ describe Docker::Image do
       let(:opts) { {} }
       let(:block) { Proc.new {} }
       let(:container) do
-        Docker::Container.create('Image' => image.id,
-                                 'Cmd' => %w[cat /Dockerfile])
+        Docker::Container.create(
+          'Image' => image.id,
+          'Cmd' => %w[cat /Dockerfile]
+        ).tap(&:start).tap(&:wait)
       end
       let(:output) { container.streaming_logs(stdout: true) }
 
       after(:each) do
+        container.remove
         image.remove(:noprune => true)
       end
 
       context 'with no query parameters' do
-        it 'builds the image', :vcr do
-          container.tap(&:wait).start
+        it 'builds the image' do
           expect(output).to eq(docker_file.read)
         end
-
-        after do
-          container.remove
-        end
       end
 
       context 'with specifying a repo in the query parameters' do
         let(:opts) { { "t" => "#{ENV['DOCKER_API_USER']}/debian:from_dir" } }
-        it 'builds the image and tags it', :vcr do
-          container.tap(&:wait).start
+        it 'builds the image and tags it' do
           expect(output).to eq(docker_file.read)
           image.refresh!
-          expect(image.info["RepoTags"]).to eq(
-            ["#{ENV['DOCKER_API_USER']}/debian:from_dir"]
-          )
-        end
-
-        after do
-          container.remove
+          expect(image.info["RepoTags"].size).to eq(1)
+          expect(image.info["RepoTags"].first).to match(%r{#{ENV['DOCKER_API_USER']}/debian:from_dir})
         end
       end
 
@@ -601,9 +784,9 @@ describe Docker::Image do
         let(:build_output) { "" }
         let(:block) { Proc.new { |chunk| build_output << chunk } }
 
-        it 'calls the block and passes build output', :vcr do
+        it 'calls the block and passes build output' do
           image # Create the image variable, which is lazy-loaded by Rspec
-          expect(build_output).to match(/Step 0 : FROM debian:wheezy/)
+          expect(build_output).to match(/(Step|STEP) \d(\/\d)?\s?: FROM debian:stable/)
         end
 
         context 'uses a cached version the second time' do
@@ -611,9 +794,10 @@ describe Docker::Image do
           let(:block_two) { Proc.new { |chunk| build_output_two << chunk } }
           let(:image_two) { subject.build_from_dir(dir, opts, &block_two) }
 
-          it 'calls the block and passes build output', :vcr do
+          it 'calls the block and passes build output' do
+            skip('Not supported on podman') if ::Docker.podman?
             image # Create the image variable, which is lazy-loaded by Rspec
-            expect(build_output).to match(/Step 0 : FROM debian:wheezy/)
+            expect(build_output).to match(/(Step|STEP) \d(\/\d)?\s?: FROM debian:stable/)
             expect(build_output).to_not match(/Using cache/)
 
             image_two # Create the image_two variable, which is lazy-loaded by Rspec
@@ -633,8 +817,9 @@ describe Docker::Image do
         }
 
         before { Docker.creds = creds }
+        after { Docker.creds = nil }
 
-        it 'sends X-Registry-Config header', :vcr do
+        it 'sends X-Registry-Config header' do
           expect(image.info[:headers].keys).to include('X-Registry-Config')
         end
       end
