@@ -1,5 +1,7 @@
 require 'spec_helper'
 
+SingleCov.covered! uncovered: 8
+
 describe Docker do
   subject { Docker }
 
@@ -13,6 +15,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_CERT_PATH').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) { should == {} }
       its(:url) { should == 'unix:///var/run/docker.sock' }
@@ -27,6 +30,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_CERT_PATH').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) { should == {} }
       its(:url) { should == 'unixs:///var/run/not-docker.sock' }
@@ -40,6 +44,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_CERT_PATH').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) { should == {} }
       its(:url) { should == 'tcp://localhost:2375' }
@@ -54,6 +59,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_CERT_PATH').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) { should == {} }
       its(:url) { should == 'tcp://someserver:8103' }
@@ -69,6 +75,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_CERT_PATH').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) { should == {} }
       its(:url) { should == 'tcp://someotherserver:8103' }
@@ -85,6 +92,7 @@ describe Docker do
         allow(ENV).to receive(:[]).with('DOCKER_SSL_VERIFY').and_return(nil)
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) {
         should == {
@@ -109,6 +117,7 @@ describe Docker do
           .and_return('false')
         Docker.reset!
       end
+      after { Docker.reset! }
 
       its(:options) {
         should == {
@@ -136,51 +145,52 @@ describe Docker do
 
   [:options=, :url=].each do |method|
     describe "##{method}" do
-      before do
-        subject.url = nil
-        subject.options = nil
-      end
+      before { Docker.reset! }
 
       it 'calls #reset_connection!' do
         expect(subject).to receive(:reset_connection!)
-        subject.public_send(method, {})
+        subject.public_send(method, nil)
       end
     end
   end
 
   describe '#version' do
-    before do
-      subject.url = nil
-      subject.options = nil
-    end
+    before { Docker.reset! }
+
     let(:expected) {
       %w[ApiVersion Arch GitCommit GoVersion KernelVersion Os Version]
     }
 
     let(:version) { subject.version }
-    it 'returns the version as a Hash', :vcr do
+    it 'returns the version as a Hash' do
       expect(version).to be_a Hash
-      expect(version.keys.sort).to eq expected
+      expect(version.keys.sort).to include(*expected)
     end
   end
 
   describe '#info' do
-    before do
-      subject.url = nil
-      subject.options = nil
-    end
+    before { Docker.reset! }
 
     let(:info) { subject.info }
     let(:keys) do
-      %w(Containers Debug DockerRootDir Driver DriverStatus ExecutionDriver ID
-         IPv4Forwarding Images IndexServerAddress InitPath InitSha1
-         KernelVersion Labels MemTotal MemoryLimit NCPU NEventsListener NFd
-         NGoroutines Name OperatingSystem SwapLimit)
+      %w(Containers Debug DockerRootDir Driver DriverStatus ID IPv4Forwarding
+         Images IndexServerAddress KernelVersion Labels MemTotal MemoryLimit
+         NCPU NEventsListener NFd NGoroutines Name OperatingSystem SwapLimit)
     end
 
-    it 'returns the info as a Hash', :vcr do
+    it 'returns the info as a Hash' do
       expect(info).to be_a Hash
-      expect(info.keys.sort).to eq keys
+      expect(info.keys.sort).to include(*keys)
+    end
+  end
+
+  describe '#ping' do
+    before { Docker.reset! }
+
+    let(:ping) { subject.ping}
+
+    it 'returns the status as a String' do
+      expect(ping).to eq('OK')
     end
   end
 
@@ -191,13 +201,9 @@ describe Docker do
       subject.authenticate!(credentials)
     }
 
-    after do
-      Docker.creds = nil
-    end
+    after { Docker.creds = nil }
 
     context 'with valid credentials' do
-      # Used valid credentials to record VCR and then changed
-      # cassette to match these credentials
       let(:credentials) {
         {
           :username      => ENV['DOCKER_API_USER'],
@@ -207,15 +213,14 @@ describe Docker do
         }
       }
 
-      it 'logs in and sets the creds', :vcr do
+      it 'logs in and sets the creds' do
+        skip_without_auth
         expect(authentication).to be true
-        expect(Docker.creds).to eq(credentials.to_json)
+        expect(Docker.creds).to eq(MultiJson.dump(credentials))
       end
     end
 
     context 'with invalid credentials' do
-      # Recorded the VCR with these credentials
-      # to purposely fail
       let(:credentials) {
         {
           :username      => 'test',
@@ -225,8 +230,8 @@ describe Docker do
         }
       }
 
-      it "raises an error and doesn't set the creds", :vcr do
-        skip "VCR won't record when Excon::Expects fail"
+      it "raises an error and doesn't set the creds" do
+        skip('Not supported on podman') if ::Docker.podman?
         expect {
           authentication
         }.to raise_error(Docker::Error::AuthenticationError)
@@ -234,26 +239,4 @@ describe Docker do
       end
     end
   end
-
-  describe '#validate_version' do
-    before do
-      subject.url = nil
-      subject.options = nil
-    end
-
-    context 'when a Docker Error is raised' do
-      before do
-        allow(Docker).to receive(:info).and_raise(Docker::Error::ClientError)
-      end
-
-      it 'raises a Version Error' do
-        expect { subject.validate_version! }
-            .to raise_error(Docker::Error::VersionError)
-      end
-    end
-
-    context 'when nothing is raised', :vcr do
-      its(:validate_version!) { should be true }
-    end
-  end
 end
