@@ -1,8 +1,14 @@
+require 'bundler/setup'
+
 $LOAD_PATH.unshift(File.join(File.dirname(__FILE__), '..', 'lib'))
 
-require 'rspec'
 require 'rspec/its'
-require 'simplecov'
+
+require 'single_cov'
+
+# avoid coverage failure from lower docker versions not running all tests
+SingleCov.setup :rspec
+
 require 'docker'
 
 ENV['DOCKER_API_USER']  ||= 'debbie_docker'
@@ -15,6 +21,12 @@ RSpec.shared_context "local paths" do
   end
 end
 
+module SpecHelpers
+  def skip_without_auth
+    skip "Disabled because of missing auth" if ENV['DOCKER_API_USER'] == 'debbie_docker'
+  end
+end
+
 Dir["#{File.dirname(__FILE__)}/support/**/*.rb"].each { |f| require f }
 
 RSpec.configure do |config|
@@ -22,4 +34,5 @@ RSpec.configure do |config|
   config.color = true
   config.formatter = :documentation
   config.tty = true
+  config.include SpecHelpers
 end
